@@ -6,7 +6,7 @@ from uuid import uuid4
 import random
 
 # module settings
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 __all__ = [
     'is_string',
     'is_url',
@@ -18,11 +18,14 @@ __all__ = [
     'is_uuid',
     'is_ip',
     'words_count',
+    'contains_html',
     'camel_case_to_snake',
     'snake_case_to_camel',
     'reverse',
     'uuid',
     'shuffle',
+    'strip_html',
+    'prettify',
 ]
 
 # compiled regex
@@ -55,10 +58,86 @@ CREDIT_CARDS = {
     'DISCOVER': re.compile(r'^6(?:011|5[0-9]{2})[0-9]{12}$'),
     'JCB': re.compile(r'^(?:2131|1800|35\d{3})\d{11}$')
 }
-JSON_WRAPPER_RE = re.compile(r'^\s*\{\s*(.|\s)*\s*\}\s*$', re.MULTILINE)
+JSON_WRAPPER_RE = re.compile(r'^\s*\{\s*.*\s*\}\s*$', re.MULTILINE | re.DOTALL)
 UUID_RE = re.compile(r'^[a-f\d]{8}-[a-f\d]{4}-[a-f\d]{4}-[a-f\d]{4}-[a-f\d]{12}$', re.IGNORECASE)
 IP_RE = re.compile(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$')
 WORDS_COUNT_RE = re.compile(r'\W*[^\W_]+\W*', re.IGNORECASE | re.MULTILINE | re.UNICODE)
+HTML_RE = re.compile(
+    r'((<([a-z]+:)?[a-z]+[^>]*/?>)(.*?(</([a-z]+:)?[a-z]+>))?|<!--.*-->|<!doctype.*>)',
+    re.IGNORECASE | re.MULTILINE | re.DOTALL
+)
+HTML_TAG_ONLY_RE = re.compile(
+    r'(<([a-z]+:)?[a-z]+[^>]*/?>|</([a-z]+:)?[a-z]+>|<!--.*-->|<!doctype.*>)',
+    re.IGNORECASE | re.MULTILINE | re.DOTALL
+)
+PRETTIFY_RE = {
+    # match repetitions of signs that should not be repeated (like multiple spaces or duplicated quotes)
+    'DUPLICATES': re.compile(
+        r'(\({2,}|\){2,}|\[{2,}|\]{2,}|\{{2,}|\}{2,}|:{2,}|,{2,}|;{2,}|\+{2,}|\-{2,}|\s{2,}|%{2,}|={2,}|"{2,}|\'{2,})',
+        re.MULTILINE
+    ),
+    # check that a sign cannot have a space before or missing a space after,
+    # unless it is a dot or a comma, where numbers may follow (5.5 or 5,5 is ok)
+    'RIGHT_SPACE': re.compile(
+        r'('
+        r'(?<=[^\s\d]),(?=[^\s\d])|\s,\s|\s,(?=[^\s\d])|\s,(?!.)|'  # comma (,)
+        r'(?<=[^\s\d\.])\.+(?=[^\s\d\.])|\s\.+\s|\s\.+(?=[^\s\d])|\s\.+(?!\.)|'  # dot (.)
+        r'(?<=\S);(?=\S)|\s;\s|\s;(?=\S)|\s;(?!.)|'  # semicolon (;)
+        r'(?<=\S):(?=\S)|\s:\s|\s:(?=\S)|\s:(?!.)|'  # colon (:)
+        r'(?<=[^\s!])!+(?=[^\s!])|\s!+\s|\s!+(?=[^\s!])|\s!+(?!!)|'  # exclamation (!)
+        r'(?<=[^\s\?])\?+(?=[^\s\?])|\s\?+\s|\s\?+(?=[^\s\?])|\s\?+(?!\?)|'  # question (?)
+        r'\d%(?=\S)|(?<=\d)\s%\s|(?<=\d)\s%(?=\S)|(?<=\d)\s%(?!.)'  # percentage (%)
+        r')',
+        re.MULTILINE | re.DOTALL
+    ),
+    'LEFT_SPACE': re.compile(
+        r'('
+
+        # quoted text ("hello world")
+        r'\s"[^"]+"(?=[\?\.:!,;])|(?<=\S)"[^"]+"\s|(?<=\S)"[^"]+"(?=[\?\.:!,;])|'
+
+        # text in round brackets
+        r'\s\([^\)]+\)(?=[\?\.:!,;])|(?<=\S)\([^\)]+\)\s|(?<=\S)(\([^\)]+\))(?=[\?\.:!,;])'
+
+        r')',
+        re.MULTILINE | re.DOTALL
+    ),
+    # match chars that must be followed by uppercase letters (like ".", "?"...)
+    'UPPERCASE_AFTER_SIGN': re.compile(
+        r'([\.\?!]\s\w)',
+        re.MULTILINE | re.UNICODE
+    ),
+    'SPACES_AROUND': re.compile(
+        r'('
+        r'(?<=\S)\+(?=\S)|(?<=\S)\+\s|\s\+(?=\S)|'  # plus (+)
+        r'(?<=\S)\-(?=\S)|(?<=\S)\-\s|\s\-(?=\S)|'  # minus (-)
+        r'(?<=\S)/(?=\S)|(?<=\S)/\s|\s/(?=\S)|'  # division (/)
+        r'(?<=\S)\*(?=\S)|(?<=\S)\*\s|\s\*(?=\S)|'  # multiplication (*)
+        r'(?<=\S)=(?=\S)|(?<=\S)=\s|\s=(?=\S)|'  # equal (=)
+
+        # quoted text ("hello world")
+        r'\s"[^"]+"(?=[^\s\?\.:!,;])|(?<=\S)"[^"]+"\s|(?<=\S)"[^"]+"(?=[^\s\?\.:!,;])|'
+
+        # text in round brackets
+        r'\s\([^\)]+\)(?=[^\s\?\.:!,;])|(?<=\S)\([^\)]+\)\s|(?<=\S)(\([^\)]+\))(?=[^\s\?\.:!,;])'
+
+        r')',
+        re.MULTILINE | re.DOTALL
+    ),
+    'SPACES_INSIDE': re.compile(
+        r'('
+        r'(?<=")[^"]+(?=")|'  # quoted text ("hello world")
+        r'(?<=\()[^\)]+(?=\))'  # text in round brackets
+        r')',
+        re.MULTILINE | re.DOTALL
+    ),
+    'SAXON_GENITIVE': re.compile(
+        r'('
+        r'(?<=\w)\'\ss\s|(?<=\w)\s\'s(?=\w)|(?<=\w)\s\'s\s(?=\w)'
+        r')',
+        re.MULTILINE | re.UNICODE
+    )
+}
 
 
 # string checking functions
@@ -90,7 +169,7 @@ def is_url(string, allowed_schemes=None):
     :rtype: bool
     """
     try:
-        valid = bool(URL_RE.match(string))
+        valid = bool(URL_RE.search(string))
     except TypeError:
         return False
     if allowed_schemes:
@@ -118,7 +197,7 @@ def is_email(string):
     :rtype: bool
     """
     try:
-        return bool(EMAIL_RE.match(string))
+        return bool(EMAIL_RE.search(string))
     except TypeError:
         return False
 
@@ -154,9 +233,9 @@ def is_credit_card(string, card_type=None):
                 raise KeyError(
                     'Invalid card type "%s". Valid types are: %s' % (card_type, ', '.join(CREDIT_CARDS.keys()))
                 )
-            return bool(CREDIT_CARDS[card_type].match(string))
+            return bool(CREDIT_CARDS[card_type].search(string))
         for c in CREDIT_CARDS:
-            if CREDIT_CARDS[c].match(string):
+            if CREDIT_CARDS[c].search(string):
                 return True
     except TypeError:
         return False
@@ -179,7 +258,7 @@ def is_camel_case(string):
     :rtype: bool
     """
     try:
-        return bool(CAMEL_CASE_TEST_RE.match(string))
+        return bool(CAMEL_CASE_TEST_RE.search(string))
     except TypeError:
         return False
 
@@ -209,7 +288,7 @@ def is_snake_case(string, separator='_'):
     re_template = '^[a-z]+([a-z\d]+{sign}|{sign}[a-z\d]+)+[a-z\d]+$'
     r = re_map.get(separator, re.compile(re_template.format(sign=re.escape(separator))))
     try:
-        return bool(r.match(string))
+        return bool(r.search(string))
     except TypeError:
         return False
 
@@ -224,7 +303,7 @@ def is_json(string):
     :rtype: bool
     """
     s = str(string)
-    if bool(JSON_WRAPPER_RE.match(s)):
+    if bool(JSON_WRAPPER_RE.search(s)):
         try:
             return isinstance(json.loads(s), dict)
         except (TypeError, ValueError, OverflowError):
@@ -241,7 +320,7 @@ def is_uuid(string):
     :return: True if UUID, false otherwise
     :rtype: bool
     """
-    return bool(UUID_RE.match(str(string)))
+    return bool(UUID_RE.search(str(string)))
 
 
 def is_ip(string):
@@ -254,7 +333,7 @@ def is_ip(string):
     :rtype: bool
     """
     try:
-        return bool(IP_RE.match(string))
+        return bool(IP_RE.search(string))
     except TypeError:
         return False
 
@@ -276,6 +355,20 @@ def words_count(string):
     return len(WORDS_COUNT_RE.findall(string))
 
 
+def contains_html(string):
+    """
+    Checks if the given string contains html code.
+    By design, this function is very permissive regarding what to consider html code, don't expect to use it
+    as an html validator, its goal is to detect "malicious" or undesired html tags in the text.
+
+    :param string: Text to check
+    :type string: str
+    :return: True if string contains html, false otherwise.
+    :rtype: bool
+    """
+    return bool(HTML_RE.search(string))
+
+
 # string manipulation functions
 
 def reverse(string):
@@ -360,3 +453,76 @@ def shuffle(string):
     s = sorted(string)  # turn the string into a list of chars
     random.shuffle(s)  # shuffle the list
     return ''.join(s)  # convert the shuffled list back to string
+
+
+def strip_html(string, keep_tag_content=False):
+    """
+    Remove html code contained into the given string.
+
+    :param string: String to manipulate.
+    :type string: str
+    :param keep_tag_content: True to preserve tag content, False to remove tag and its content too (default).
+    :type keep_tag_content: bool
+    :return: String with html removed.
+    :rtype: str
+    """
+    r = HTML_TAG_ONLY_RE if keep_tag_content else HTML_RE
+    return r.sub('', string)
+
+
+def prettify(string):
+    """
+    Turns an ugly text string into a beautiful one by applying a regex pipeline which ensures the following:
+
+    - String cannot start or end with spaces
+    - String cannot have multiple sequential spaces, empty lines or punctuation (except for "?", "!" and ".")
+    - Arithmetic operators ("+", "-", "/", "*", "=") must have one, and only one space before and after themselves
+    - The first letter after a dot, an exclamation or a question mark must be uppercase
+    - One, and only one space should follow a dot, an exclamation or a question mark
+    - Text inside double quotes cannot start or end with spaces, but one, and only one space must come first and \
+    after quotes (foo" bar"baz -> foo "bar" baz)
+    - Text inside round brackets cannot start or end with spaces, but one, and only one space must come first and \
+    after brackets ("foo(bar )baz" -> "foo (bar) baz")
+    - Percentage sign ("%") cannot be preceded by a space if there is a number before ("100 %" -> "100%")
+    - Saxon genitive is correct ("Dave' s dog" -> "Dave's dog")
+
+
+    :param string: String to manipulate
+    :return: Prettified string.
+    :rtype: str
+    """
+
+    def remove_duplicates(regex_match):
+        return regex_match.group(1)[0]
+
+    def uppercase_first_letter_after_sign(regex_match):
+        match = regex_match.group(1)
+        return match[:-1] + match[2].upper()
+
+    def ensure_right_space_only(regex_match):
+        return regex_match.group(1).strip() + ' '
+
+    def ensure_left_space_only(regex_match):
+        return ' ' + regex_match.group(1).strip()
+
+    def ensure_spaces_around(regex_match):
+        return ' ' + regex_match.group(1).strip() + ' '
+
+    def remove_internal_spaces(regex_match):
+        return regex_match.group(1).strip()
+
+    def fix_saxon_genitive(regex_match):
+        return regex_match.group(1).replace(' ', '') + ' '
+
+    p = PRETTIFY_RE['DUPLICATES'].sub(remove_duplicates, string)
+    p = PRETTIFY_RE['RIGHT_SPACE'].sub(ensure_right_space_only, p)
+    p = PRETTIFY_RE['LEFT_SPACE'].sub(ensure_left_space_only, p)
+    p = PRETTIFY_RE['SPACES_AROUND'].sub(ensure_spaces_around, p)
+    p = PRETTIFY_RE['SPACES_INSIDE'].sub(remove_internal_spaces, p)
+    p = PRETTIFY_RE['UPPERCASE_AFTER_SIGN'].sub(uppercase_first_letter_after_sign, p)
+    p = PRETTIFY_RE['SAXON_GENITIVE'].sub(fix_saxon_genitive, p)
+    p = p.strip()
+    try:
+        return p[0].capitalize() + p[1:]
+    except IndexError:
+        return p
